/*
kissdo2.c  bare bones disciplined oscillator  with faster lock  version 0.1


    Copyright (C) 2022 Marko Cebokli   S57UUU
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program. If not, see <https://www.gnu.org/licenses/>. 

compile:
msp430-gcc -Os -Wall -g -mmcu=msp430g2553 kissdo2.c -o kissdo2.elf
proram MCU:
mspdebug rf2500 "prog kissdo2.elf"

Pin  3	RXD	TTY
Pin  4	TXD	TTY
Pin  9	TA1.1	1PPS input from GPS
Pin 13	TA1.2	PWM output coarse
Pin 14	TA0.1	PWM output fine
Pin 19	XIN	10MHz input from OCXO


*/

#include  <msp430.h>
#include <in430.h>

#include "beri_pisi.h"

//global variables

unsigned long pwm;
int dd,d,oldd,dda;
int dt,pwc,pwf,flg;	//for TTY output
int dum;
int n1,lastdd;
int nrog;

//***********************************************************
int main(void)
{

WDTCTL = WDTPW + WDTHOLD;       // Stop WDT
BCSCTL1 = CALBC1_16MHZ;         // Set DCO for SMCLK
DCOCTL = CALDCO_16MHZ;

BCSCTL3 = LFXT1S_3;		//ext clock
__bic_SR_register(OSCOFF);	//osc off=0
BCSCTL2 = SELM_3;		//MCLK=XT1    doesnt work? 

P2DIR = 0x25;		//P2.0 = TA1.0 out  P2.2=SW debug out  P2.5 = PWM out
P2SEL |= 0x23;		//P2.0 = TA1.0 out  P2.1 = TA1.1 input  P2.5 = PWM out

TA0CTL = TASSEL_2 + MC_1;	//SMCLK, upmode
TA0CCR0 = 63;			//period  4us
TA0CCR1 = 31;			//PWM fine initial value
TA0CCTL1 = OUTMOD_7;		//reset/set, PWM

TA1CTL = TASSEL_1 + MC_1;	//ACLK, upmode
TA1CCR0 = 9999;
TA1CCTL0 = OUTMOD_4; 		//toggle, 500Hz out
TA1CCTL1 = CM_1 + CAP + CCIS_0 + CCIE;	//capture rising, CCI1A=pin9, irq enable
TA1CCR2 = 5599;			//PWM coarse
TA1CCTL2 = OUTMOD_7;		//reset/set, PWM

UCA0CTL1 |= UCSSEL_2;       // SMCLK
UCA0BR0 = 104;              // 16MHz 9600 bps
UCA0BR1 = 0;                // 16MHz 9600 bps
UCA0MCTL = UCBRS_0+UCBRF_3+UCOS16;       // Modulation UCBRSx = 1
UCA0CTL1 &= ~UCSWRST;       // Initialize USCI state machine
//IE2 |= UCA0RXIE;          // Enable USCI_A0 RX interrupt

P1SEL = BIT1 + BIT2 + BIT6; // P1.1 = RXD, P1.2 = TXD, P1.6 = TA0.1
P1SEL2 = BIT1 + BIT2;       // P1.1 = RXD, P1.2 = TXD
P1DIR |= 0x44;		    // P1.2 = TXD, P1.6 (TA0.1) = Fine PWM

pwm=5555*65536;
pwm=0;
oldd=0;
flg=0;
n1=0; lastdd=0;
nrog=0;

__enable_interrupt();


while(1)
  {
  while (flg==0)
    {
//    P2OUT ^= 0x04;	//debug output P2.2 pin 10
    dum=0;	//an empty loop hangs???
    }
  poslji_string("   D= ");poslji16Sdec(d,0);	//send some data to tty
  poslji_string("   DD= ");poslji16Sdec(dd,0);
  poslji_string("   Pc= ");poslji16Sdec(pwc,0);
  poslji_string("   Pf= ");poslji16Sdec(pwf,0);
  poslji_string("   n1= ");poslji16Sdec(n1,0);
  poslji_string("   nr= ");poslji16Sdec(nrog,0);
  poslji_crlf();
  flg=0;
//  P2OUT ^= 0x04;	//debug output P2.2 pin 10
  }

return 0;
}


//-----------------------------------------------------------------
//Timer irq
#pragma vector=TIMER1_A1_VECTOR
__interrupt void ta1_ISR(void)
{
  
//convert timer value into signed time difference
if (TA1CCR1 < 5000) d=TA1CCR1; else d=TA1CCR1-10000;

//time derivative of time differece = frequency difference
dd = d - oldd;
oldd = d;
if (dd > 5000) dd-=10000;	//fix rollover
if (dd < -5000) dd+=10000;

if (dd==0)
  {
  n1++; if (n1>10000) n1=10000;
  if (nrog >0) nrog--;
  }
else
  {
  dda = (dd>=0) ? dd : -dd;	//abs()
  if ((dda==1)&&(dd == -lastdd)) {lastdd=dd; flg=1; return;}
  lastdd=dd;
  if (dda > 30) return;		//ignore rogue pulses
  
  if (dda > 1) 	//NOTE:  loop gains based on a 10Hz/V OCXO EFC (CTI OSC5A2B02)
    {
    if (nrog<10) nrog++;
    if (nrog>3)
      pwm = pwm - 1310720*(long)dd;	//dd based loop G=20.0 => 1310720   long
//      pwm = pwm - 32768*(long)dd;	//dd based loop G=5.0 => 32768   long
    }
  else		//dda==1
    {
    if (nrog==0)
      {
      if (n1 < 15)
	pwm = pwm - 1310728*(long)dd;		//fixed step cca 12mHz (abs(dd)==1)
      else
	pwm = pwm - 7000000/(long)n1*(long)dd;	//n1 based loop
//	pwm = pwm - 5000000/(long)n1*(long)dd;	//n1 based loop
//	pwm = pwm - 2000000/(long)n1*(long)dd;	//n1 based loop
      n1=0;
      }
    else nrog--;
    }
    
//  n1=0;
  }
  
if (pwm < 5*65536) pwm=5*65536;	//keep within timer range
if (pwm > 9995*65536) pwm=9995*65536;

TA1CCR2 = pwm>>16;			//coarse PWM
TA0CCR1 = (pwm & 0x0000FC00) >> 10;	//fine PWM

pwc=pwm>>16;	//for printout
pwf=(pwm>>10) & 0x003F;

flg=1;		//flag for printout
//P2OUT ^= 0x04;	//debug output P2.2 pin 10

TA1CCTL1 &= ~CCIFG;     //reset irq flag
}
